#[repr(C)]pub struct DrawIndexedIndirect {
pub index_count: u32,
pub instance_count: u32,
pub first_index: u32,
pub base_vertex: i32,
pub first_instance: u32,
}Expand description
Indirect draw command for indexed geometry.
This matches the layout expected by wgpu::RenderPass::draw_indexed_indirect.
Fields
index_count- Number of indices to drawinstance_count- Number of instances to drawfirst_index- Index of the first index to drawbase_vertex- Value added to each index before indexing into the vertex bufferfirst_instance- Instance ID of the first instance (requires INDIRECT_FIRST_INSTANCE)
Fields
index_count: u32instance_count: u32first_index: u32base_vertex: i32first_instance: u32Implementations
Sourceimpl DrawIndexedIndirect
impl DrawIndexedIndirect
Sourcepub const fn new(
index_count: u32,
instance_count: u32,
first_index: u32,
base_vertex: i32,
first_instance: u32,
) -> Self
pub const fn new( index_count: u32, instance_count: u32, first_index: u32, base_vertex: i32, first_instance: u32, ) -> Self
Create a new indexed indirect draw command.
Sourcepub const fn single(index_count: u32) -> Self
pub const fn single(index_count: u32) -> Self
Create a simple indexed draw command for a single instance.