pub struct Material { /* private fields */ }Expand description
A material manages shader parameters, textures, and pipeline state.
Implementations
Sourceimpl Material
impl Material
Sourcepub fn new(shader: Arc<ShaderModule>, context: Arc<GraphicsContext>) -> Self
pub fn new(shader: Arc<ShaderModule>, context: Arc<GraphicsContext>) -> Self
Create a new material with a shader.
Sourcepub fn from_source(
source: &str,
label: Option<&str>,
context: Arc<GraphicsContext>,
) -> Self
pub fn from_source( source: &str, label: Option<&str>, context: Arc<GraphicsContext>, ) -> Self
Create a material from a shader source string.
Sourcepub fn set_parameter(
&mut self,
name: impl Into<String>,
value: MaterialParameter,
)
pub fn set_parameter( &mut self, name: impl Into<String>, value: MaterialParameter, )
Set a parameter by name.
Sourcepub fn get_parameter(&self, name: &str) -> Option<&MaterialParameter>
pub fn get_parameter(&self, name: &str) -> Option<&MaterialParameter>
Get a parameter by name.
Sourcepub fn set_texture(&mut self, name: impl Into<String>, texture: MaterialTexture)
pub fn set_texture(&mut self, name: impl Into<String>, texture: MaterialTexture)
Set a texture by name.
Sourcepub fn get_texture(&self, name: &str) -> Option<&MaterialTexture>
pub fn get_texture(&self, name: &str) -> Option<&MaterialTexture>
Get a texture by name.
Sourcepub fn set_pipeline_state(&mut self, state: PipelineState)
pub fn set_pipeline_state(&mut self, state: PipelineState)
Set the pipeline state.
Sourcepub fn pipeline_state(&self) -> &PipelineState
pub fn pipeline_state(&self) -> &PipelineState
Get the pipeline state.
Sourcepub fn shader(&self) -> &ShaderModule
pub fn shader(&self) -> &ShaderModule
Get the shader module.
Sourcepub fn bind<'a>(&'a mut self, pass: &mut RenderPass<'a>, bind_group_index: u32)
pub fn bind<'a>(&'a mut self, pass: &mut RenderPass<'a>, bind_group_index: u32)
Bind this material’s resources to a render pass.
This will update GPU resources if needed and set the bind group.
Arguments
pass- The render pass to bind tobind_group_index- The bind group index (default is usually 0)
Sourcepub fn bind_group_layout(&mut self) -> &BindGroupLayout
pub fn bind_group_layout(&mut self) -> &BindGroupLayout
Get the bind group layout (creates it if needed).
This is useful when creating render pipelines.