pub struct StagingBufferPool { /* private fields */ }Expand description
A pool of staging buffers for reuse.
Implementations
Sourceimpl StagingBufferPool
impl StagingBufferPool
Sourcepub fn allocate(
&mut self,
context: &GraphicsContext,
size: u64,
) -> StagingBuffer
pub fn allocate( &mut self, context: &GraphicsContext, size: u64, ) -> StagingBuffer
Allocate a staging buffer from the pool.
If a suitable buffer is available, it will be reused. Otherwise, a new buffer will be created.
Sourcepub fn recycle(&mut self, buffer: StagingBuffer)
pub fn recycle(&mut self, buffer: StagingBuffer)
Return a staging buffer to the pool for reuse.
Sourcepub fn available_count(&self) -> usize
pub fn available_count(&self) -> usize
Get the number of available buffers.
Sourcepub fn total_available_size(&self) -> u64
pub fn total_available_size(&self) -> u64
Get the total size of available buffers.