pub struct WindowContextDescriptor {
pub format: Option<TextureFormat>,
pub present_mode: Option<PresentMode>,
pub alpha_mode: Option<CompositeAlphaMode>,
pub with_depth: bool,
pub depth_format: Option<TextureFormat>,
}Expand description
Descriptor for configuring a window’s rendering context.
Fields
format: Option<TextureFormat>The surface texture format. If None, uses the default format for the surface.
present_mode: Option<PresentMode>Present mode for the surface.
alpha_mode: Option<CompositeAlphaMode>Alpha mode for the surface.
with_depth: boolWhether to create a depth texture for this window.
When enabled, the window will maintain an auto-resizing depth texture
that can be accessed via Frame::depth_view().
depth_format: Option<TextureFormat>The depth texture format. Defaults to Depth32Float if not specified.
Only used when with_depth is true.