AsWgpuMut

Trait AsWgpuMut 

pub trait AsWgpuMut: AsWgpu {
    // Required method
    fn as_wgpu_mut(&mut self) -> &mut Self::WgpuType;
}
Access the underlying wgpu type (mutable).

Implement this trait to expose mutable access to the underlying wgpu type.

Required Methods

fn as_wgpu_mut(&mut self) -> &mut Self::WgpuType

Get a mutable reference to the underlying wgpu type.

Implementors