RenderPassExt

Trait RenderPassExt 

pub trait RenderPassExt {
    // Required methods
    fn clear_pass<'a>(
        &'a mut self,
        target: RenderTarget<'a>,
        clear_color: Color,
    ) -> RenderPass<'a>;
    fn load_pass<'a>(&'a mut self, target: RenderTarget<'a>) -> RenderPass<'a>;
}
Helper trait for creating render passes with common configurations.

Required Methods

fn clear_pass<'a>( &'a mut self, target: RenderTarget<'a>, clear_color: Color, ) -> RenderPass<'a>

Create a render pass that clears to the given color.

fn load_pass<'a>(&'a mut self, target: RenderTarget<'a>) -> RenderPass<'a>

Create a render pass that loads existing content.

Implementors