pub type UniformBuffer<T> = TypedBuffer<T>;Expand description
A uniform buffer for shader uniforms.
This is a convenience type for buffers that contain uniform data.
Aliased Type
pub struct UniformBuffer<T> { /* private fields */ }pub type UniformBuffer<T> = TypedBuffer<T>;A uniform buffer for shader uniforms.
This is a convenience type for buffers that contain uniform data.
pub struct UniformBuffer<T> { /* private fields */ }