pub struct GraphicsContextDescriptor {
pub backends: Backends,
pub power_preference: PowerPreference,
pub force_fallback_adapter: bool,
pub required_gpu_features: GpuFeatures,
pub requested_gpu_features: GpuFeatures,
pub additional_wgpu_features: Features,
pub limits: Limits,
pub label: Option<&'static str>,
}Expand description
Descriptor for configuring graphics context creation.
Fields
backends: BackendsGPU backends to use
power_preference: PowerPreferencePower preference for adapter selection
force_fallback_adapter: boolWhether to force fallback adapter
required_gpu_features: GpuFeaturesRequired GPU features (panics if not available).
Use this for features that your application cannot function without.
requested_gpu_features: GpuFeaturesRequested GPU features (best-effort, logs warning if unavailable).
Use this for features that would be nice to have but are not essential.
additional_wgpu_features: FeaturesAdditional raw wgpu features to enable (for features not covered by GpuFeatures).
limits: LimitsRequired device limits
label: Option<&'static str>Optional label for debugging
Implementations
Sourceimpl GraphicsContextDescriptor
impl GraphicsContextDescriptor
Sourcepub fn require_features(self, features: GpuFeatures) -> Self
pub fn require_features(self, features: GpuFeatures) -> Self
Set required GPU features (panics if not available).
Sourcepub fn request_features(self, features: GpuFeatures) -> Self
pub fn request_features(self, features: GpuFeatures) -> Self
Set requested GPU features (best-effort, warns if unavailable).
Sourcepub fn with_required_features(self, features: GpuFeatures) -> Self
pub fn with_required_features(self, features: GpuFeatures) -> Self
Add additional required features.
Sourcepub fn with_requested_features(self, features: GpuFeatures) -> Self
pub fn with_requested_features(self, features: GpuFeatures) -> Self
Add additional requested features.
Sourcepub fn with_wgpu_features(self, features: Features) -> Self
pub fn with_wgpu_features(self, features: Features) -> Self
Set additional raw wgpu features (for features not covered by GpuFeatures).
Sourcepub fn power_preference(self, preference: PowerPreference) -> Self
pub fn power_preference(self, preference: PowerPreference) -> Self
Set the power preference.