pub struct IndirectBuffer<T: IndirectCommand> { /* private fields */ }Expand description
A type-safe GPU buffer for indirect draw commands.
This wrapper ensures type safety and provides convenient methods for writing and using indirect draw commands.
Type Parameters
T- The type of indirect command (eitherDrawIndirectorDrawIndexedIndirect)
Example
ⓘ
use astrelis_render::{IndirectBuffer, DrawIndexedIndirect, Renderer};
// Create an indirect buffer for 100 indexed draw commands
let indirect_buffer = IndirectBuffer::<DrawIndexedIndirect>::new(
context,
Some("My Indirect Buffer"),
100,
);
// Write commands
let commands = vec![
DrawIndexedIndirect::single(36), // Draw 36 indices
DrawIndexedIndirect::instanced(36, 10), // Draw 36 indices, 10 instances
];
indirect_buffer.write(&context.queue, &commands);
// In render pass
render_pass.draw_indexed_indirect(indirect_buffer.buffer(), 0);Implementations
Sourceimpl<T: IndirectCommand> IndirectBuffer<T>
impl<T: IndirectCommand> IndirectBuffer<T>
Sourcepub fn size_bytes(&self) -> u64
pub fn size_bytes(&self) -> u64
Get the size of the buffer in bytes.
Sourcepub fn offset_of(&self, index: usize) -> u64
pub fn offset_of(&self, index: usize) -> u64
Get the byte offset of a command at the given index.