pub struct RingBuffer { /* private fields */ }Expand description
A ring buffer for streaming per-frame data.
Ring buffers cycle through multiple frames worth of buffer space to avoid stalling the GPU pipeline.
Implementations
Sourceimpl RingBuffer
impl RingBuffer
Sourcepub fn new(
context: Arc<GraphicsContext>,
size: u64,
usage: BufferUsages,
) -> Self
pub fn new( context: Arc<GraphicsContext>, size: u64, usage: BufferUsages, ) -> Self
Create a new ring buffer.
Arguments
context- Graphics contextsize- Total size in bytes (will be multiplied by RING_BUFFER_FRAMES)usage- Buffer usage flags (UNIFORM, STORAGE, etc.)
Sourcepub fn next_frame(&mut self)
pub fn next_frame(&mut self)
Advance to the next frame.
Call this at the beginning or end of each frame to reset the ring buffer for the next frame’s allocations.