pub trait RenderPassExt {
// Required methods
fn clear_pass<'a>(
&'a mut self,
target: RenderTarget<'a>,
clear_color: Color,
) -> RenderPass<'a>;
fn load_pass<'a>(&'a mut self, target: RenderTarget<'a>) -> RenderPass<'a>;
}Expand description
Helper trait for creating render passes with common configurations.
Required Methods
Sourcefn clear_pass<'a>(
&'a mut self,
target: RenderTarget<'a>,
clear_color: Color,
) -> RenderPass<'a>
fn clear_pass<'a>( &'a mut self, target: RenderTarget<'a>, clear_color: Color, ) -> RenderPass<'a>
Create a render pass that clears to the given color.
Sourcefn load_pass<'a>(&'a mut self, target: RenderTarget<'a>) -> RenderPass<'a>
fn load_pass<'a>(&'a mut self, target: RenderTarget<'a>) -> RenderPass<'a>
Create a render pass that loads existing content.